package Friends

import (
	"encoding/json"
	"io/ioutil"
	"net/http"

	"git.tovijaeschke.xyz/tovi/Envelope/Backend/Database"
	"git.tovijaeschke.xyz/tovi/Envelope/Backend/Models"
	"git.tovijaeschke.xyz/tovi/Envelope/Backend/Util"
)

func Friend(w http.ResponseWriter, r *http.Request) {
	var (
		userData   Models.User
		returnJson []byte
		err        error
	)

	userData, err = Util.GetUserById(w, r)
	if err != nil {
		http.Error(w, "Not Found", http.StatusNotFound)
		return
	}

	returnJson, err = json.MarshalIndent(userData, "", "  ")
	if err != nil {
		http.Error(w, "Error", http.StatusInternalServerError)
		return
	}

	w.WriteHeader(http.StatusOK)
	w.Write(returnJson)
}

func CreateFriendRequest(w http.ResponseWriter, r *http.Request) {
	var (
		friendRequest Models.FriendRequest
		requestBody   []byte
		returnJson    []byte
		err           error
	)

	requestBody, err = ioutil.ReadAll(r.Body)
	if err != nil {
		panic(err)
	}

	err = json.Unmarshal(requestBody, &friendRequest)
	if err != nil {
		panic(err)
	}

	err = Database.CreateFriendRequest(&friendRequest)
	if err != nil {
		panic(err)
	}

	returnJson, err = json.MarshalIndent(friendRequest, "", "  ")
	if err != nil {
		http.Error(w, "Error", http.StatusInternalServerError)
		w.WriteHeader(http.StatusInternalServerError)
		return
	}

	// Return updated json
	w.WriteHeader(http.StatusOK)
	w.Write(returnJson)
}
